
% Illustrates the color wheel tool for visualizing complex valued functions
% (see also https://en.wikipedia.org/wiki/Domain_coloring)
% by plotting a few analytic functions.
%
% Color wheels visualize complex numbers using colors, where the phase of
% the complex number is encoded in the hue, and the magnitude encoded in
% the brightness and saturation.
%
% Typically, color wheels are centered at z=0.  However, as the complex
% damping parameters we are interested lie in the disk |z-1|<=1, we
% center our color wheel at z=1.
%
% Please note that the color wheel visualization as we have designed it
% rescales the color so that maximum saturation and brightness are assigned
% to the point maximizing |f(z)-1|.  This works well for bounded
% functions, which suits us well since the damping parameters we are
% interested in are confined to the disk |z-1|<=1.
%
% This file contains a number of examples that we hope will serve as a
% useful reference when examining colorwheel plots of the spatially varying
% complex damping parameters generated by our other files. Running the
% program will result in three figures.
%
% Figure 1 provides a plot the identity map f(z)=z.  This
% allows you to see the distribution of colors in the complex plane.  In
% particular z=1 is mapped to black, real numbers greater than 1 are
% colored cyan, and real numbers smaller than 1 are colored red.  Denoting
% i :=sqrt(-1), we have +i colored magenta while -i is colored orange.
%
% Figure 2 visualizes the function f(z)=sqrt(z-1), which has
% a branch cut along (-infty,1].  This branch cut is clearly visible as a
% jump in color from magenta to orange, as the value of the square root
% jumps from the positive imaginary axis to the negative imaginary axis.
%
% Figure 3 plot f(z)=e^z.  Notice the periodicity along the
% imaginary axis.

addpath ComplexVisualization

h = 0.01;

x = 0:h:2;
y = -1:h:1;
[X,Y]=meshgrid(x,y);
Z = X+1i*Y;

center = 1;
contours=0;
colorWheel(X,Y,Z,center,contours);
xlabel('x')
ylabel('iy')
title('color wheel visualization of f(z)=z, centered at z = 1, no isocontours')

center = 1;
contours=1;
colorWheel(X,Y,(Z-1).^0.5,center,contours);
xlabel('x')
ylabel('iy')
title('color wheel visualization of f(z)=sqrt(z-1), centered at z = 1, with isocontours')

x = -1:h:2;
y = -2*pi:h*pi:2*pi;
[X,Y]=meshgrid(x,y);
Z = X+1i*Y;

center = 1;
contours=1;
colorWheel(X,Y,exp(Z),center,contours);
xlabel('x')
ylabel('iy')
title('color wheel visualization of f(z)=exp(z), centered at z = 1, with isocontours')