
% This file will allow you to reproduce results from the Marmousi Experiment
% in Section 6.6 of the paper.  An appropriate "setup" object is created
% and passed into "Multigrid.m".  Type "help Multigrid" for an explanation
% of what the parameters of the setup object do.  
%
% We fix the strategy as 'opt-complex' for this experiment.  Changing to a
% different strategy will cause an error, as this is the only strategy
% supported with variable coefficients.
%
% Please download the universal lookup table at 
% https://www.cs.ubc.ca/~greif/Publications/hg2020_code/UniversalLookup.mat
% and put it in the current directory
% in order to run this experiment with RB-SOR.

clear all;
close all;

% setup.coefficients must be one of "constant", "variable", or "Marmousi".
% For this experiment, we use Marmousi.

setup.coefficients = 'Marmousi';

% setup.strategy must be one of 
%
% 1. opt-complex
% 2. opt-real
% 3. baseline
% 4. scan
% 5. two-grid
% 6. omega_ub
%
% Type "Help dampingParameterStrategyExperiment" in the command line to
% find out in detail what these options do.
%
% For variable coefficient problems (which includes Marmousi), 
% only 'opt-complex' is
% supported.  Using a different strategy will result in an error.

setup.strategy='opt-complex';

% rhs should be 'random' or 'point_source'.  We reported 'random' in the
% paper, but you can also try out 'point_source'.

setup.rhs = 'point_source';

% for a random RHS we average over 10 iterations.  But feel free to change
% this.

setup.numRandomIt=10;

% type should be 'vertex-centered' or 'cell-centered'.  We reported
% cell-centered in the paper.  We have observed vertex-centered multigrid
% to be less stable for this problem - converging in some cases but not in
% others (dependent on the mesh resolution).

setup.type = 'cell-centered'; 

% smoother must be one of 'DAMPED-JACOBI' or 'RB-SOR'.

setup.smoother='RB-SOR';

setup.nu = 2; % for RB-SOR, should be to 1 or 2.  For DAMPED-JACOBI, can be any positive integer.

% setup.epsilon doesn't do anything for Marmousi.  We set it to 0.

setup.epsilon = 0.0;

% controls the maximum value that kh will take on (for constant coefficient
% problems, like this one, it is also the value kh takes on everywhere).
% Set to pi/5 for this experiment.

setup.kh_M = pi/5;

% the amount of damping - set to 0.5 by default (but we also try 0.4 in
% this experiment).

setup.alpha = 0.5;

% For this experiment, the dimension must be two, as the Marmousi dataset
% is two dimensional.  Setting dimension equal to three will cause an
% error.

setup.dimension = 2;

% The grid has N = 2^level - delta interior grid cells along each
% dimension.  For vertex-centered multigrid, delta = 1.  For cell-centered,
% delta = 0.

% in the paper, we tried level = 7,8,9,10,11,11 and compared convergence rates,
% runtimes, etc.  You can try a bigger level if you want, but be prepared
% to wait for a long time.

setup.level = 8;

% if set to one, multigrid will plot the rhs and the solution (in the 2D
% case - in the 3D case it will plot a representative slice).  If set to
% zero, multigrid will not do this.  For this experiment, the Marmousi
% dataset will also be plotted.

setup.plotSurfaces = 1;

% if a spatially varying damping parameter is used, a plot of its real and
% imaginary part on the top "dampingParameterGraphsNum" grids will be
% plotted.  So, if setup.dampingParameterGraphsNum=0, no graphs will be
% plotted.  If setup.dampingParameterGraphsNum=1, only the spatially
% varying damping parameter on the finest grid will be plotted.  If 
% setup.dampingParameterGraphsNum=2, the spatially varying damping parameter
% on the finest grid and the first coarse grid will be plotted, and so on.

setup.dampingParameterGraphsNum = 3;

% by default, we visualize our spatially-varying complex damping parameters
% using a colorwheel (see ColorWheelDemo.m).  Set to 0 to get the real and
% imaginary parts as separate surfaces instead.

setup.useColorWheel = 1;

% since we are dealing with a damped wave equation, typically only a 
% few wavelengths are visible.  One way to combat this is to raise the
% solution to the power of a number close to zero, counteracting the decay
% of the waves and making more wavelengths visible.

setup.increaseWaveVisibility=1;

[u,res_vec]=Multigrid(setup);