
function [u,res_vec]=Multigrid(setup)

% This help file makes references to equation numbers in the paper 
% "Optimal Complex Damping Parameters Minimizing Red-Black SOR Multigrid Smoothing Factors for Complex-Shifted Linear Systems"
%
%     Input comes in the form of one structure, called "setup", whose fields are:
% 
% 1.  setup.dimension.  Number of space dimensions. Must be equal to 2 or 3 (1 is not allowed/supported)
% 
% 2.  setup.type.  Must be 'vertex-centered' or 'cell-centered'.
% 
% 3.  setup.smoother.  Must be one of 'RB-SOR' or 'DAMPED-JACOBI'.
% 
% 4.  setup.nu.  The number of smoothing sweeps. For 'RB-SOR', only nu=1,2 are currently supported.  For 'DAMPED-JACOBI', nu can be any positive integer.
% 
% 5.  setup.level.  Our problem has N = 2^level-1 (vertex-centered) or N=2^level (cell-centered) interior grid points
% along each axis on the finest grid (except in the case of Marmousi, where we have roughly three times as many points along the x-axis as the y-axis).
% 
% 6.  setup.epsilon.  Should be [0,1] (purpose discussed shortly)
% 
% 7.  setup.kh_M.   The maximum value of kh, defined in Equation (6.3)-(6.4) of the paper.  Is set to sqrt(epsilon)*pi/5, or simply pi/5, by default.
% 
% 8.  setup.alpha.  The damping of the medium (0.5 by default).
% 
% 9.  setup.rhs.  The right hand side vector.  Must be "random" or "point_source".
% 
% 10. setup.numRandomIt.  If a random right hand side is used, multigrid is run numRandomIt times and the results are averaged.  Set to ten by default.
% 
% 11. setup.plotSurfaces.  Should be 0 or 1.  If it is 1, surfaces of rhs and solution will be plotted (in the case of Marmousi, k(x,y) will also be plotted).
%
% 12. setup.dampingParameterGraphsNum.  If a spatially varying damping parameter is used, a plot of its real and
% imaginary part on the top "dampingParameterGraphsNum" grids will be
% plotted.  So, if setup.dampingParameterGraphsNum=0, no graphs will be
% plotted.  If setup.dampingParameterGraphsNum=1, only the spatially
% varying damping parameter on the finest grid will be plotted.  If 
% setup.dampingParameterGraphsNum=2, the spatially varying damping parameter
% on the finest grid and the first coarse grid will be plotted, and so on.
% In the case of constant coefficients, this parameter does nothing and
% need not be set.
%
% 13. setup.useColorWheel.  If set to 1, any spatially varying complex
% damping parameters will be visualized using a color wheel (see
% ColorWheelDemo.m and its help).  If it set to 0, the real and
% imaginary parts of the complex damping parameters will be separately
% plotted as surfaces.
%
% 14. setup.increaseWaveVisibility.  Should be 0 or 1.  If it is 1, the solution will be rescaled in order to make decaying waves easier to see.
% 
% 15.  setup.coefficients.  Must be one of "constant", "variable", or "Marmousi".  If "constant" is selected, 
% it is assumed that we are dealing with the stencil (6.1), with 
%     i.epsilon_1 = epsilon, epsilon_2 = 2-epsilon (if the dimension is 2)
% 	ii. epsilon_1 = epsilon, epsilon_2 = epsilon_3 = (3-epsilon)/2 (if the dimension is 3)
% If you want to use a different (constant) stencil, as long as it is of the form (1.5), you can do so.  The routine
% getL_2D(a,c1,c1,c2,c2,N) and getL_3D(a,c1,c1,c2,c2,c3,c3,N) will generate the necessary matrices.  The only place
% these routines are called is in the "getOperators" in the "MultigridComponents" directory.
% 
% If "variable" is selected, then you will end up with the stencil (6.1) but with each epsilon_j now a function of position, namely 
% epsilon_1 = 1+epsilon*sin(2*pi*x)
% epsilon_2 = 1+epsilon*sin(2*pi*y)
% epsilon_3 = 1+epsilon*sin(2*pi*y)   (if the dimension is three)
% You can try a different variable coefficient problem simply by changing what "getL_variable" does.  
% The spatially varying damping parameters will be generated for you automatically because they are found based on extracting each row of the
% output of "getL_variable".
% 
% If "Marmousi" is selected, then you end up with the istropic Helmholtz equation (2.3), with k(x,y) given by the Marmousi dataset illustrated in Figure 6.
% This option cannot be run in 3D, because the Marmousi dataset is 2D.  Setup.epsilon is ignored for this problem.
% 
% 16.  setup.strategy.  Must be one of the following:
% 
%     i. opt-complex
%      - find optimal complex damping parameters seperately on all
%        levels (recommended)
% 
%     ii. opt-real
%      - find optimal real damping parameters seperately on all
%        levels
% 
%     iii. baseline
%      - uses omega = 1 for RB-SOR and omega = 0.5 for DAMPED-JACOBI on all
%        levels
% 
%     iv. scan
%      - uses exhaustive scanning to find a single optimal complex damping
%        parameter used on all levels
%        (be prepared to wait a very long time if you use this strategy)
% 
%     v. two-grid
%      - computes optimal 2-grid damping parameters on every level
%        (only compatible with vertex-centered multigrid in two dimensions
%         be prepared to wait a very long time if you use this strategy)
% 
%     vi. omega_ub
% 
%      - uses omega_ub (Theorem 5.3) as a proxy of omega_opt on all levels
%        (only compatible with RB-SOR smoothing).
% 	   
% Note that for variable coefficients and Marmousi, only the opt-complex strategy is supported.  Moreover, the two-grid strategy is only compatible with
% vertex-centered multigrid in two dimensions.  
%
% Note that for variable coefficients and Marmousi, you must download 
% the universal lookup table at 
% https://www.cs.ubc.ca/~greif/Publications/hg2020_code/UniversalLookup.mat 
% and put it in the current directory in order to run RB-SOR.
%
% This code returns the solution vector u and the residual convergence
% history res_vec.



    addpath MultigridComponents
    addpath SmoothingTheory
    addpath TwoGridTheory
    addpath Marmousi
    addpath ComplexVisualization

    % check that the smoother is one of 'RB-SOR' or 'DAMPED-JACOBI'
    if ~strcmp(setup.smoother,'RB-SOR') && ~strcmp(setup.smoother,'DAMPED-JACOBI')
        error('Invalid smoother: must be one of "RB-SOR" or "DAMPED-JACOBI"')
    end
    % check that setup.type is one of 'vertex-centered' or 'cell-centered'
    if ~strcmp(setup.type,'vertex-centered') && ~strcmp(setup.type,'cell-centered')
        error('Invalid setup.type: must be one of "vertex-centered" or "cell-centered"')
    end
    % check that setup.strategy is a valid choice
    if ~strcmp(setup.strategy,'opt-complex') && ~strcmp(setup.strategy,'opt-real') && ~strcmp(setup.strategy,'baseline') && ~strcmp(setup.strategy,'scan') && ~strcmp(setup.strategy,'two-grid') && ~strcmp(setup.strategy,'omega_ub')
        error('Invalid setup.strategy')
    end
    % check that setup.rhs is a valid choice
    if ~strcmp(setup.rhs,'random') && ~strcmp(setup.rhs,'point_source')
        error('Invalid rhs: must be one of "random" or "point_source"')
    end
    
    
    if ~strcmp(setup.coefficients,'constant') && strcmp(setup.smoother,'RB-SOR')
    
        % try to load the universal lookup table.
        try
            load UniversalLookup.mat Lookup
            setup.Lookup = Lookup;
        catch
        % if it is not found, remind the user to download it.
            error('Variable coefficient problems using RB-SOR require the universal lookup table, which is not found.  Please download it at https://www.cs.ubc.ca/~greif/Publications/hg2020_code/UniversalLookup.mat and place it in the current directory');
        end
        
    end
    
    if ~strcmp(setup.coefficients,'constant') && ~strcmp(setup.strategy,'opt-complex')
       error('For non-constant coefficients, only the opt-complex strategy is supported') 
    end
    
    if strcmp(setup.coefficients,'constant')                 
        setup.minLevel = 1; % bottom grid in the heirarchy with have N_bottom = 2^minLevel - delta...
    else
        setup.minLevel = 4;
    end

    if strcmp(setup.type,'vertex-centered')
        setup.delta = 1;
    elseif strcmp(setup.type,'cell-centered')
        setup.delta = 0;
    else
        error('invalid setup type')
    end

    % we work on the unit cube except in the case of Marmousi.

    if ~strcmp(setup.coefficients,'Marmousi')
        setup.domainHeight = 1.0;
    else
        setup.domainHeight = 3.0;
    end

    % Our grid has N interior points on each dimension and is uniform with mesh
    % spacing h.  N and h will be slightly different depending on if we are
    % using vertex-centered or cell-centered multigrid.

    N = 2^(setup.level)-setup.delta;
    h = setup.domainHeight/(N+setup.delta);
    d = setup.dimension;
    
    % For constant coefficients we have kh <= sqrt(epsilon)*(pi/5) to meet
    % accuracy constraints.  For variable coefficients we take kh = pi/5.
    % For Marmousi, kh is no longer constant, but its maximum is  pi/5.

    if strcmp(setup.coefficients,'constant') || strcmp(setup.coefficients,'variable')
        setup.K = setup.kh_M/h;   
    elseif strcmp(setup.coefficients,'Marmousi')
        setup.K_max = (setup.kh_M)/h;  
    else
        error('invalid coefficient type')
    end

    % let's get pre- and post-smoothing sweep numbers nu1 and nu2.
    [nu1,nu2]=getNu1Nu2(setup);
    setup.nu1 = nu1;
    setup.nu2 = nu2;

            %%% basic parameters are now set %%%


    % Now let's precompute all the matrices we need to run multigrid.
    % Ls is a cell array consisting of the system matrices on all levels
    % Ps is a cell array consisting of the prolongation operators on all levels
    % Rs is a cell array consisting of the restriction operators on all levels

    [Ls,Ps,Rs]=getOperators(setup);
    
    % In the case of RB-SOR, multigrid will run faster if we precompute the
    % block 2x2 red-black ordering L = [D_r, L_rb ; L_br, D_b] 
    % of every matrix L in Ls, as well as a
    % few other things including the permutation matrices taking you back and
    % forth between lexicographic and red-black orderings.

    setup = getRBStuff(Ls,setup);
    
    % It's now time to get our damping parameters.  How we do this depends on
    % if we are working with constant or variable coefficients.

    if strcmp(setup.coefficients,'constant')

        % extracts the stencil data [a,c,c_m] on each level (needed for theoretical smoothing
        % factor computation).  This only works for constant coefficient stencils.

        setup.stencil_data = getStencilData(Ls,setup);

        % in this case we compute one damping parameter per level based on
        % the strategy in question and the stencil data just extracted.  
        % Do not use this method for variable coefficient problems.

        [dampingParameters,smoothingFactors] = getDampingParametersConstantCoefficient(Ls,Ps,Rs,setup);

        setup.dampingParameters = dampingParameters;
        setup.smoothingFactors = smoothingFactors;

    elseif strcmp(setup.coefficients,'variable') || strcmp(setup.coefficients,'Marmousi')

        % in this case we will automatically extract the stencil components of
        % every row of every matrix in the multigrid hierarchy, creating
        % vectors a, c, and c_m which are then used to create spatially varying
        % damping parameters, using either the universal lookup table (RB-SOR)
        % or our explicit formula (damped-Jacobi)
        setup.Ws = getDampingParameterMatrices(Ls,setup);

    end

    % if we have spatially varying damping parameters, we need to
    % pre-compute their W = [Wrr, Wbb] form with red-black ordering.
    
    if strcmp(setup.smoother,'RB-SOR') && ~strcmp(setup.coefficients,'constant')
    
        setup = permuteOmegaMatrices(setup);
        
    end
    
    
    
    %%%% lets prepare to run multigrid %%%%%
    
    setup.tol = 1e-10; % tolerance for multigrid convergence to be declared.
    setup.maxR = 1e6; % tolerance for multigrid divergence to be declared.
    setup.maxIT = 5000; % maximum number of multigrid iterations.
    
    
    % If the righthand side is random, we will run multigrid with
    % M=10 random right hand sides, and average the
    % converge rates.  Plots will only be given for the final run.
    
    % Otherwise, if the rhs is a point source we will run multigrid M = 1 time.

    if strcmp(setup.rhs,'random')
        M = setup.numRandomIt;
    elseif strcmp(setup.rhs,'point_source')
        M = 1;
    else
        error('invalid rhs');
    end

    multigrid_rates = zeros(M,1);
    multigrid_iterations = zeros(M,1);
    multigrid_runtimes = zeros(M,1);
    
    for random_run = 1:M

        % generate rhs
        if strcmp(setup.rhs,'random')
            f = getRandomRHS(Ls,setup,random_run);
        elseif strcmp(setup.rhs,'point_source') 
            f = getPointRHS(setup);
        end

        if ~strcmp(setup.coefficients,'Marmousi')
            u = zeros(N^d,1); % initial guess of zeros
        else
            u = zeros(N*(3*N+2*setup.delta),1);
        end
        
        L = Ls{setup.level};
        
        r = f-L*u; % initial residual
        r0 = norm(r); % norm of the initial residual.

        iter_count = 1; % iteration count

        res_vec = [1]; % initial normalized residual norm is equal to 1 by construction.

        % we need to keep track of how many times we've visited each level in order
        % to perform an F-cycle properly.
        setup.levelCounts = zeros(setup.level,1);

        tic

        % multigrid main loop

        disp(['**** running iteration ',num2str(random_run),' of ',num2str(M),' of multigrid ****']); 
        
        while sqrt(r'*r)/r0 > setup.tol && iter_count < setup.maxIT && sqrt(r'*r)/r0 < setup.maxR

            disp(['iter_count=',num2str(iter_count),' ||r|| ',num2str(sqrt(r'*r)/r0)]);

            u = FCYC(Ls,Ps,Rs,u,f,setup.level,setup);
            setup.levelCounts = zeros(setup.level,1);
            r = f-L*u;
            iter_count=iter_count+1;
            res_vec(iter_count) = (r'*r)^0.5/r0;

        end

        multigrid_runtimes(random_run) = toc;

        if random_run == M

            figure
            semilogy(res_vec,'LineWidth',2)
            title('multigrid convergence plot')
            xlabel('iterations')
            ylabel('norm of residual')
            xlim([1 iter_count])
            set(gca,'fontsize',20,'LineWidth',2)

        end

        %nu=1;

        res_reduction_vec=[];

        for ii=2:iter_count
            res_reduction_vec(ii)=(res_vec(ii)/res_vec(ii-1));
        end

        if random_run == M
            
            figure
            plot(res_reduction_vec,'LineWidth',2)
            ylim([0 1])
            title('multigrid convergence rate')
            xlabel('iterations')
            ylabel('reduction in norm of residual')
            xlim([2 iter_count])
            set(gca,'fontsize',20,'LineWidth',2)
            
            if setup.plotSurfaces
                plotSolution(u,setup)
            end

        end

        multigrid_rates(random_run)=getMultigridRate(res_reduction_vec);
        multigrid_iterations(random_run) = iter_count-1; % subtract one since iter_count is initialized at 1

    end

    printMultigridData(u,multigrid_rates,multigrid_iterations,multigrid_runtimes,setup)




