% Computes the optimal damping parameters based on numerical minimization,
% as described in Section 5.3 of the paper, as opposed to using the
% universal lookup table as described in Section 5.4.  This method should
% only be used for constant coefficient problems.

function [dampingParameters,smoothingFactors] = getDampingParametersConstantCoefficient(Ls,Ps,Rs,setup)

    % we need the dimension to equal to 2 or 3, or this will break.  First
    % check the dimension and error if it is wrong.
    
    d = setup.dimension;
    nu = setup.nu;
    
    if d~=2 && d~=3
        error('dimension should be equal to two or three')
    end

    dampingParameters=zeros(setup.level,1);
    smoothingFactors=zeros(setup.level,1);
    
    % in the scanning strategy, we find a single parameter that is best for
    % all levels, rather than going level by level separately.
    if strcmp(setup.strategy,'scan')
        [w_scan,~] = manuallyFindOptimalDampingParameterOverC(Ls,Ps,Rs,setup);
    end
    
    % we find the optimal parameters for all grids but the lowest in the
    % multigrid hierarchy, where the system will be inverted directly.
    % (or, if we used the scanning strategy, we just flug "w_scan" into the
    % smoothing factor formulas on all grids).
    
    for l = setup.minLevel+1:setup.level
        
        % the first step is to extract the stencil for the present level.  
        stencil_data_l = setup.stencil_data{l};
        a = stencil_data_l(1);
        c = stencil_data_l(2);
        c_m = stencil_data_l(3);
        
        if strcmp(setup.strategy,'opt-complex') || strcmp(setup.strategy,'opt-real')
            if strcmp(setup.smoother,'DAMPED-JACOBI')
                [wR,wC,muR,muC ] = DampedJacobiSmoothing(a,c,c_m);
                if strcmp(setup.strategy,'opt-complex')
                    dampingParameters(l)=wC;
                    smoothingFactors(l)=muC;
                elseif strcmp(setup.strategy,'opt-real')
                    dampingParameters(l)=wR;
                    smoothingFactors(l)=muR;
                end
            elseif strcmp(setup.smoother,'RB-SOR')
                if strcmp(setup.strategy,'opt-complex')
                    [w,mu]=optimalRB_SOR(a,c,c_m,nu,'C');
                    dampingParameters(l)=w;
                    smoothingFactors(l)=mu;
                elseif strcmp(setup.strategy,'opt-real')
                    [w,mu]=optimalRB_SOR(a,c,c_m,nu,'R');
                    dampingParameters(l)=w;
                    smoothingFactors(l)=mu;
                end
            end
        elseif strcmp(setup.strategy,'baseline')
            if strcmp(setup.smoother,'DAMPED-JACOBI')
                dampingParameters(l)=0.5;
                b1 = 1-2*(c-c_m)/a;
                b2 = 1+2*c/a;
                smoothingFactors(l)=max([abs(1-0.5*b1),abs(1-0.5*b2)]);
            elseif strcmp(setup.smoother,'RB-SOR')
                dampingParameters(l)=1;
                smoothingFactors(l)=RBSOR_SmoothingFactor(a,c,c_m,nu,1);
            end
        elseif strcmp(setup.strategy,'omega_ub')
            if strcmp(setup.smoother,'DAMPED-JACOBI')
                error('omega_ub not defined for DAMPED-JACOBI');
            else
                % Theorem 5.3
                w_ub = 2/(1+sqrt(1-4*(c-c_m)^2/a^2));
                dampingParameters(l)=w_ub;
                smoothingFactors(l)=RBSOR_SmoothingFactor(a,c,c_m,nu,w_ub);
            end
        elseif strcmp(setup.strategy,'scan')
            if strcmp(setup.smoother,'DAMPED-JACOBI')
                dampingParameters(l)=w_scan;
                b1 = 1-2*(c-c_m)/a;
                b2 = 1+2*c/a;
                smoothingFactors(l)=max([abs(1-w_scan*b1),abs(1-w_scan*b2)]);
            elseif strcmp(setup.smoother,'RB-SOR')
                dampingParameters(l)=w_scan;
                smoothingFactors(l)=RBSOR_SmoothingFactor(a,c,c_m,nu,w_scan);
            end
        elseif strcmp(setup.strategy,'two-grid')
            
            if ~setup.dimension==2
                error('two-grid optimization only supported in two dimensions')
            elseif strcmp(setup.type,'cell-centered')
                error('two-grid optimization does not support cell-centered discretizations')
            end
            
            % find the optimal two-grid parameter, using the optimal LFA
            % parameter as an initial guess.
            
            if strcmp(setup.smoother,'RB-SOR')
                [w0,~]=optimalRB_SOR(a,c,c_m,nu,'C');
            elseif strcmp(setup.smoother,'DAMPED-JACOBI')
                [~,w0,~,~] = DampedJacobiSmoothing(a,c,c_m);
            end
            
            N = 2^l-setup.delta;
            h = setup.domainHeight/(N+setup.delta);
            kh = setup.K*h;
            
            % create two-grid setup object
            twoGridSetup = buildDirectShiftedLaplacianSetup(setup.alpha,setup.epsilon,kh,setup.nu1,setup.nu2,w0,setup.smoother);
            [wC,rho2G]=optimal2G(twoGridSetup,w0);
            
            dampingParameters(l)=wC;
            % in this case, the "smoothing factor" is really the nuth root
            % of the two-grid spectral radius.
            smoothingFactors(l)=rho2G^(1/setup.nu);

        else
            error('invalid strategy')
        end
        
        
        
    end
    
end
    
    