function values=interpolate_lookup_table(c_m,r,phi,Lookup,lookupValue)

% c_m, r, and phi are vectors of the same length.  We want to perform
% trilinear interpolation using the lookup table "Lookup", and we want to
% do so in a vectorized way, so that Matlab doesn't take forever.
% Unfortunately this will be a bit convoluted.

[M,N,~,~]=size(Lookup);

% Lookup is M x N x N x 4, with Lookup(:,:,:,1) and Lookup(:,:,:,2)
% containing the optimal damping parameters and associated smoothing factors
% for nu = 1, while Lookup(:,:,:,3) and Lookup(:,:,:,4) contain the same
% for nu = 2.  The parameter "lookupValue" tells us what we are currently
% interested in.

if strcmp(lookupValue,'omega1')
    Lookup_flat = reshape(Lookup(:,:,:,1),[M*N^2,1]);
elseif strcmp(lookupValue,'mu1')
    Lookup_flat = reshape(Lookup(:,:,:,2),[M*N^2,1]);
elseif strcmp(lookupValue,'omega2')   
    Lookup_flat = reshape(Lookup(:,:,:,3),[M*N^2,1]);
elseif strcmp(lookupValue,'mu2')
    Lookup_flat = reshape(Lookup(:,:,:,4),[M*N^2,1]);
else
    error('invalid lookupValue')
end

Rmax = 2;
Cm_max = 0.25;
PHImax = pi;

% Our lookup table only considers 0<=phi<=pi.  In reality, phi can also
% take values in -pi<phi<0.  But in this case, the effect will just be to
% complex conjugate omega and to leave mu alone.  
% So we keep track of which values of phi are
% negative, then find omega or mu associated with abs(phi), 
% and later complex conjugate omega associated with the phi that were 
% originally negative (if we are looking for omega), or do nothing (if we are
% looking for mu).  But since mu is real, complex conjugating it is the
% same as doing nothing, so we just always complex conjugate.

phi_negative = phi<0;
phi = abs(phi);

h_r = Rmax/(M-1);
h_c = Cm_max/(N-1);
h_phi = PHImax/(N-1);

u1 = floor(1+r/h_r);
v1 = floor(1+c_m/h_c);
w1 = floor(1+phi/h_phi);

u2 = ceil(1+r/h_r);
v2 = ceil(1+c_m/h_c);
w2 = ceil(1+phi/h_phi);

% if r>2, u2 will be outside the range of the lookup table.  In that case,
% we just set omega to 1 (this is justified, one can show that for r >> 1,
% the optimal omega converges to 1.  This was omitted from the paper
% because of space concerns).  In any event, we need to keep track of which
% indices lie outside of the lookup table in order to avoid an error.

valid_indices = u2<=M;

u = 1+r/h_r - u1;
v = 1+c_m/h_c - v1;
w = 1+phi/h_phi - w1;

% to prevent an out of bounds error, we set all invalid indices equal to 1,
% but we remember that we did so.

u1(~valid_indices)=1;
u2(~valid_indices)=1;
u(~valid_indices)=0;

v1(~valid_indices)=1;
v2(~valid_indices)=1;
v(~valid_indices)=0;

w1(~valid_indices)=1;
w2(~valid_indices)=1;
w(~valid_indices)=0;

idx_111 = sub2ind([M,N,N],u1,v1,w1);
idx_222 = sub2ind([M,N,N],u2,v2,w2);

idx_122 = sub2ind([M,N,N],u1,v2,w2);
idx_212 = sub2ind([M,N,N],u2,v1,w2);
idx_221 = sub2ind([M,N,N],u2,v2,w1);

idx_112 = sub2ind([M,N,N],u1,v1,w2);
idx_121 = sub2ind([M,N,N],u1,v2,w1);
idx_211 = sub2ind([M,N,N],u2,v1,w1);

% perform trilinear interpolation.
    
values = u.*v.*w.*Lookup_flat(idx_222)+(1-u).*v.*w.*Lookup_flat(idx_122)+u.*(1-v).*w.*Lookup_flat(idx_212)+u.*v.*(1-w).*Lookup_flat(idx_221)+(1-u).*(1-v).*w.*Lookup_flat(idx_112)+(1-u).*v.*(1-w).*Lookup_flat(idx_121)+u.*(1-v).*(1-w).*Lookup_flat(idx_211)+(1-u).*(1-v).*(1-w).*Lookup_flat(idx_111);

% We now have to deal with those (r,phi,c_m) triples with r outside the
% range of the table (that is, r>2).  Since omega --> 1 as r --> infty,
% if we are looking for omega, we just set it equal to 1 if r>2

if strcmp(lookupValue,'omega1') || strcmp(lookupValue,'omega2')
    values(~valid_indices)=1;
else
    % this means we're after the smoothing factor mu.  Since mu --> 0 as
    % r--> infty, we just set mu = 0 in this case.  This is only an
    % approximation.
    values(~valid_indices)=0;
end

% next, for those values of phi that were negative, we need to complex
% conjugate omega

values(phi_negative)=conj(values(phi_negative));

