
% This file will allow you to reproduce results from the 2D constant-coefficient
% Experiment in Sections 6.2-6.3 of the paper, the purpose of which is to 
% compare different damping parameter selection strategies for multigrid with RB-SOR
% or damped Jacobi smoothing.
%
% An appropriate "setup" object is created
% and passed into "Multigrid.m".  Type "help Multigrid" for an explanation
% of what the parameters of the setup object do.  
%
% Note that you do not need the universal lookup table to run this
% experiment.
%
%% strategy should be one of... 
%
% opt-complex
%     - find optimal complex damping parameters seperately on all
%       levels (recommended)
%
% opt-real
%     - find optimal real damping parameters seperately on all
%       levels
%
% baseline
%     - uses omega = 1 for RB-SOR and omega=0.5 for DAMPED-JACOBI on all
%       levels
%
% scan
%     - uses exhaustive scanning to find a single optimal complex damping
%       parameter used on all levels
%       (be prepared to wait a very long time if you use this strategy)
%
% two-grid
%     - computes optimal 2-grid damping parameters on every level
%       (only compatible with vertex-centered multigrid in two dimensions
%        be prepared to wait a very long time if you use this strategy)
%
% omega_ub
%
%     - uses omega_ub (Theorem 5.3) as a proxy of omega_opt on all levels
%       (only compatible with RB-SOR smoothing).

clear all;
close all;

setup.strategy='opt-complex';

% setup.coefficients must be one of "constant", "variable", or "Marmousi".
% For this experiment, we must have constant coefficients (variable
% coefficients only support the opt-complex strategy).

setup.coefficients = 'constant';

% rhs should be 'random' or 'point_source'.  We reported 'random' in the
% paper, but you can also try out 'point_source'.

setup.rhs = 'random';

% for a random RHS we average over 10 iterations.  But feel free to change
% this.

setup.numRandomIt=10;

% type should be 'vertex-centered' or 'cell-centered'.  We reported the
% former in the paper, but feel free to try the latter.

setup.type = 'vertex-centered'; 

% smoother must be one of 'DAMPED-JACOBI' or 'RB-SOR'.

setup.smoother='RB-SOR';

setup.nu = 1; % for RB-SOR, should be to 1 or 2.  For DAMPED-JACOBI, can be any positive integer.

% we reported results for epsilon = 1, 1/3 and 1/10 in the paper.  But
% you can make epsilon anything in (0,1].

setup.epsilon = 1;

% controls the maximum value that kh will take on (for constant coefficient
% problems, like this one, it is also the value kh takes on everywhere).
% Set to sqrt(epsilon)*pi/5 by default, in order to be as large as possible
% while satisfying the constraint (6.4) in the paper.

setup.kh_M = sqrt(setup.epsilon)*pi/5;

% the amount of damping - set to 0.5 by default.

setup.alpha = 0.5;

% the dimension must be two or three.  We reported the case d=2 in the
% paper, but this experiment can also be run with d=3.

setup.dimension = 2;

% The grid has N = 2^level - delta interior grid cells along each
% dimension.  For vertex-centered multigrid, delta = 1.  For cell-centered,
% delta = 0.

setup.level = 8; % set to 8 by default.

% if set to one, multigrid will plot the rhs and the solution (in the 2D
% case - in the 3D case it will plot a representative slice).  If set to
% zero, multigrid will not do this.

setup.plotSurfaces = 1;

% since we are dealing with a damped wave equation, typically only a 
% few wavelengths are visible.  One way to combat this is to raise the
% solution to the power of a number close to zero, counteracting the decay
% of the waves and making more wavelengths visible.

setup.increaseWaveVisibility=0;

[u,res_vec]=Multigrid(setup);