% This file is set up to solve
%
% -a(x,y)*u_xx-b(x,y)*u_yy-k^2*(1-1i*alpha)*u = f   (d=2)
%
% or
%
% -a(x,y,z)*u_xx-b(x,y,z)*u_yy-c(x,y,z)*u_zz-k^2*(1-1i*alpha)*u = f   (d=3)
%
% Where k is constant.  By default, we have
%
% a = 1+epsilon*sin(2*pi*x)
% b = 1+epsilon*sin(2*pi*y)
% c = 1+epsilon*sin(2*pi*z)  (if you set the dimension to three),
%
% which is the variable coefficient problem from Section 6.5 of the paper. 
% If you would like to try out different functions a,b,c, simply
% change the formulas defined below in setup.a, setup.b, and setup.c.
% Note that a,b,c should be real and
% strictly positive.  Multigrid convergence should deteriorate if these
% functions take on values close to zero.
%
% This routine requires the universal lookup table if run with RB-SOR
% smoothing, which is recommended.  Therefore, if you have not already
% done so, please download it at 
% https://www.cs.ubc.ca/~greif/Publications/hg2020_code/UniversalLookup.mat
% and put it in the current directory.
%
% Like our other experiments, we begin by creating a multigrid setup
% object, and then running "Multigrid" with setup as a parameter.
% We fix setup.strategy as 'opt-complex' for this experiment.  Changing to a
% different strategy will cause an error, as this is the only strategy
% supported with variable coefficients.


clear all;
close all;

% setup.epsilon is used in the formulas for the functions a,b and c below.
% We reported results for epsilon = 0.5, 0.7 and 0.9 in the paper.  But
% you can make epsilon anything in [0,1).

setup.epsilon = 0.7;

% The three lines of code below define the functions a,b,c as 
% string formulas.  Please note that x,y,z
% below are vectors, and the functions a,b,c should be defined accordingly.
% For example, if we want 
%    a(x,y)=1+epsilon*sin(2*pi*x)*sin(2*pi*y)
%    b(x,y)=1+epsilon*cos(2*pi*x)*cos(2*pi*y)
% we would use the two commented out lines below (note the .*):
%    setup.a = '1+epsilon*sin(2*pi*x).*sin(2*pi*y)';
%    setup.b = '1+epsilon*cos(2*pi*x).*cos(2*pi*y)';
% Note that the epsilon in these formulas is set using setup.epsilon, and
% note that setup.c only needs to be defined (and only gets used) if
% setup.dimension = 3.

setup.a = '1+epsilon*sin(2*pi*x)';
setup.b = '1+epsilon*sin(2*pi*y)';
setup.c = '1+epsilon*sin(2*pi*z)';

% setup.coefficients must be one of "constant", "variable", or "Marmousi".
% For this experiment, we use variable coefficients.

setup.coefficients = 'variable';

% setup.strategy must be one of 
%
% 1. opt-complex
% 2. opt-real
% 3. baseline
% 4. scan
% 5. two-grid
% 6. omega_ub
%
% Type "Help dampingParameterStrategyExperiment" in the command line to
% find out in detail what these options do.
%
% For variable coefficient problems, only 'opt-complex' is
% supported.  Using a different strategy will result in an error.

setup.strategy='opt-complex';

% rhs should be 'random' or 'point_source'.  We reported 'random' in the
% paper, but you can also try out 'point_source'.

setup.rhs = 'point_source';

% for a random RHS we average over 10 iterations.  But feel free to change
% this.

setup.numRandomIt=10;

% type should be 'vertex-centered' or 'cell-centered'.  We reported the
% former in the paper, but feel free to try the latter.

setup.type = 'vertex-centered'; 

% smoother must be one of 'DAMPED-JACOBI' or 'RB-SOR'.

setup.smoother='RB-SOR';

setup.nu = 2; % for RB-SOR, should be to 1 or 2.  For DAMPED-JACOBI, can be any positive integer.

% controls the maximum value that kh will take on (for constant coefficient
% problems, like this one, it is also the value kh takes on everywhere).
% Set to pi/5 in this experiment.

setup.kh_M = pi/5;

% the amount of damping - set to 0.5 by default.

setup.alpha = 0.5;

% the dimension must be two or three.  We reported the case d=2 in the
% paper, but this experiment can also be run with d=3.

setup.dimension = 2;

% The grid has N = 2^level - delta interior grid cells along each
% dimension.  For vertex-centered multigrid, delta = 1.  For cell-centered,
% delta = 0.

% in the paper, we tried level = 7,8,9,10,11,12 and compared convergence rates,
% runtimes, etc.  You can try a bigger level if you want, but be prepared
% to wait for a long time.

setup.level = 8;

% if set to one, multigrid will plot the rhs and the solution (in the 2D
% case - in the 3D case it will plot a representative slice).  If set to
% zero, multigrid will not do this.

setup.plotSurfaces = 1;

% if a spatially varying damping parameter is used, a plot of its real and
% imaginary part on the top "dampingParameterGraphsNum" grids will be
% plotted.  So, if setup.dampingParameterGraphsNum=0, no graphs will be
% plotted.  If setup.dampingParameterGraphsNum=1, only the spatially
% varying damping parameter on the finest grid will be plotted.  If 
% setup.dampingParameterGraphsNum=2, the spatially varying damping parameter
% on the finest grid and the first coarse grid will be plotted, and so on.

setup.dampingParameterGraphsNum = 3;

% by default, we visualize our spatially-varying complex damping parameters
% using a colorwheel (see ColorWheelDemo.m).  Set to 0 to get the real and
% imaginary parts as separate surfaces instead.

setup.useColorWheel = 1;

% since we are dealing with a damped wave equation, typically only a 
% few wavelengths are visible.  One way to combat this is to raise the
% solution to the power of a number close to zero, counteracting the decay
% of the waves and making more wavelengths visible.

setup.increaseWaveVisibility=1;

[u,res_vec]=Multigrid(setup);